'use strict';

var javascript$1 = require('@lezer/javascript');
var language = require('@codemirror/language');
var state = require('@codemirror/state');
var view = require('@codemirror/view');
var autocomplete = require('@codemirror/autocomplete');
var common = require('@lezer/common');
var lr = require('@lezer/lr');
//var highlight_js = require('.highlight/highlight.js');
var highlight = require('@jexl-parser/');

/// A collection of JavaScript-related
/// [snippets](#autocomplete.snippet).
const snippets = [
    autocomplete.snippetCompletion("function ${name}(${params}) {\n\t${}\n}", {
        label: "function",
        detail: "definition",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("for (let ${index} = 0; ${index} < ${bound}; ${index}++) {\n\t${}\n}", {
        label: "for",
        detail: "loop",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("for (let ${name} of ${collection}) {\n\t${}\n}", {
        label: "for",
        detail: "of loop",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("do {\n\t${}\n} while (${})", {
        label: "do",
        detail: "loop",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("while (${}) {\n\t${}\n}", {
        label: "while",
        detail: "loop",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("try {\n\t${}\n} catch (${error}) {\n\t${}\n}", {
        label: "try",
        detail: "/ catch block",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("if (${}) {\n\t${}\n}", {
        label: "if",
        detail: "block",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("if (${}) {\n\t${}\n} else {\n\t${}\n}", {
        label: "if",
        detail: "/ else block",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("class ${name} {\n\tconstructor(${params}) {\n\t\t${}\n\t}\n}", {
        label: "class",
        detail: "definition",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("import {${names}} from \"${module}\"\n${}", {
        label: "import",
        detail: "named",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("import ${name} from \"${module}\"\n${}", {
        label: "import",
        detail: "default",
        type: "keyword"
    })
];
/// A collection of snippet completions for TypeScript. Includes the
/// JavaScript [snippets](#lang-javascript.snippets).
const typescriptSnippets = snippets.concat([
    autocomplete.snippetCompletion("interface ${name} {\n\t${}\n}", {
        label: "interface",
        detail: "definition",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("type ${name} = ${type}", {
        label: "type",
        detail: "definition",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("enum ${name} {\n\t${}\n}", {
        label: "enum",
        detail: "definition",
        type: "keyword"
    })
]);

const cache = new common.NodeWeakMap();
const ScopeNodes = new Set([
    "Script", "Block",
    "FunctionExpression", "FunctionDeclaration", "ArrowFunction", "MethodDeclaration",
    "ForStatement"
]);
function defID(type) {
    return (node, def) => {
        let id = node.node.getChild("VariableDefinition");
        if (id)
            def(id, type);
        return true;
    };
}
const functionContext = ["FunctionDeclaration"];
const gatherCompletions = {
    FunctionDeclaration: defID("function"),
    EnumDeclaration: defID("constant"),
    VariableDefinition(node, def) { if (!node.matchContext(functionContext))
        def(node, "variable"); },
    __proto__: null
};
function getScope(doc, node) {
    let cached = cache.get(node);
    if (cached)
        return cached;
    let completions = [], top = true;
    function def(node, type) {
        let name = doc.sliceString(node.from, node.to);
        completions.push({ label: name, type });
    }
    node.cursor(common.IterMode.IncludeAnonymous).iterate(node => {
        if (top) {
            top = false;
        }
        else if (node.name) {
            let gather = gatherCompletions[node.name];
            if (gather && gather(node, def) || ScopeNodes.has(node.name))
                return false;
        }
        else if (node.to - node.from > 8192) {
            // Allow caching for bigger internal nodes
            for (let c of getScope(doc, node.node))
                completions.push(c);
            return false;
        }
    });
    cache.set(node, completions);
    return completions;
}
const Identifier = /^[\w$\xa1-\uffff][\w$\d\xa1-\uffff]*$/;
const dontComplete = [
    "TemplateString", "String", "RegExp",
    "LineComment", "BlockComment",
    "VariableDefinition", "Label",
    "PropertyDefinition", "PropertyName",
    "PrivatePropertyDefinition", "PrivatePropertyName",
    ".", "?."
];
/// Completion source that looks up locally defined names in
/// JavaScript code.
function localCompletionSource(context) {
    let inner = language.syntaxTree(context.state).resolveInner(context.pos, -1);
    if (dontComplete.indexOf(inner.name) > -1)
        return null;
    let isWord = inner.name == "VariableName" ||
        inner.to - inner.from < 20 && Identifier.test(context.state.sliceDoc(inner.from, inner.to));
    if (!isWord && !context.explicit)
        return null;
    let options = [];
    for (let pos = inner; pos; pos = pos.parent) {
        if (ScopeNodes.has(pos.name))
            options = options.concat(getScope(context.state.doc, pos));
    }
    return {
        options,
        from: isWord ? inner.from : context.pos,
        validFor: Identifier
    };
}
function pathFor(read, member, name) {
    var _a;
    let path = [];
    for (;;) {
        let obj = member.firstChild, prop;
        if ((obj === null || obj === void 0 ? void 0 : obj.name) == "VariableName") {
            path.push(read(obj));
            return { path: path.reverse(), name };
        }
        else if ((obj === null || obj === void 0 ? void 0 : obj.name) == "MemberExpression" && ((_a = (prop = obj.lastChild)) === null || _a === void 0 ? void 0 : _a.name) == "PropertyName") {
            path.push(read(prop));
            member = obj;
        }
        else {
            return null;
        }
    }
}
/// Helper function for defining JavaScript completion sources. It
/// returns the completable name and object path for a completion
/// context, or null if no name/property completion should happen at
/// that position. For example, when completing after `a.b.c` it will
/// return `{path: ["a", "b"], name: "c"}`. When completing after `x`
/// it will return `{path: [], name: "x"}`. When not in a property or
/// name, it will return null if `context.explicit` is false, and
/// `{path: [], name: ""}` otherwise.
function completionPath(context) {
    let read = (node) => context.state.doc.sliceString(node.from, node.to);
    let inner = language.syntaxTree(context.state).resolveInner(context.pos, -1);
    if (inner.name == "PropertyName") {
        return pathFor(read, inner.parent, read(inner));
    }
    else if ((inner.name == "." || inner.name == "?.") && inner.parent.name == "MemberExpression") {
        return pathFor(read, inner.parent, "");
    }
    else if (dontComplete.indexOf(inner.name) > -1) {
        return null;
    }
    else if (inner.name == "VariableName" || inner.to - inner.from < 20 && Identifier.test(read(inner))) {
        return { path: [], name: read(inner) };
    }
    else if (inner.name == "MemberExpression") {
        return pathFor(read, inner, "");
    }
    else {
        return context.explicit ? { path: [], name: "" } : null;
    }
}
function enumeratePropertyCompletions(obj, top) {
    let options = [], seen = new Set;
    for (let depth = 0;; depth++) {
        for (let name of (Object.getOwnPropertyNames || Object.keys)(obj)) {
            if (!/^[a-zA-Z_$\xaa-\uffdc][\w$\xaa-\uffdc]*$/.test(name) || seen.has(name))
                continue;
            seen.add(name);
            let value;
            try {
                value = obj[name];
            }
            catch (_) {
                continue;
            }
            options.push({
                label: name,
                type: typeof value == "function" ? (/^[A-Z]/.test(name) ? "class" : top ? "function" : "method")
                    : top ? "variable" : "property",
                boost: -depth
            });
        }
        let next = Object.getPrototypeOf(obj);
        if (!next)
            return options;
        obj = next;
    }
}
/// Defines a [completion source](#autocomplete.CompletionSource) that
/// completes from the given scope object (for example `globalThis`).
/// Will enter properties of the object when completing properties on
/// a directly-named path.
function scopeCompletionSource(scope) {
    let cache = new Map;
    return (context) => {
        let path = completionPath(context);
        if (!path)
            return null;
        let target = scope;
        for (let step of path.path) {
            target = target[step];
            if (!target)
                return null;
        }
        let options = cache.get(target);
        if (!options)
            cache.set(target, options = enumeratePropertyCompletions(target, !path.path.length));
        return {
            from: context.pos - path.name.length,
            options,
            validFor: Identifier
        };
    };
}

/// A language provider based on the [Lezer JavaScript
/// parser](https://github.com/lezer-parser/javascript), extended with
/// highlighting and indentation information.
const javascriptLanguage = language.LRLanguage.define({
    name: "javascript",
    parser: javascript$1.parser.configure({
        props: [
            language.indentNodeProp.add({
                IfStatement: language.continuedIndent({ except: /^\s*({|else\b)/ }),
                TryStatement: language.continuedIndent({ except: /^\s*({|catch\b|finally\b)/ }),
                LabeledStatement: language.flatIndent,
                SwitchBody: context => {
                    let after = context.textAfter, closed = /^\s*\}/.test(after), isCase = /^\s*(case|default)\b/.test(after);
                    return context.baseIndent + (closed ? 0 : isCase ? 1 : 2) * context.unit;
                },
                Block: language.delimitedIndent({ closing: "}" }),
                ArrowFunction: cx => cx.baseIndent + cx.unit,
                "TemplateString BlockComment": () => null,
                "Statement Property": language.continuedIndent({ except: /^{/ }),
                JSXElement(context) {
                    let closed = /^\s*<\//.test(context.textAfter);
                    return context.lineIndent(context.node.from) + (closed ? 0 : context.unit);
                },
                JSXEscape(context) {
                    let closed = /\s*\}/.test(context.textAfter);
                    return context.lineIndent(context.node.from) + (closed ? 0 : context.unit);
                },
                "JSXOpenTag JSXSelfClosingTag"(context) {
                    return context.column(context.node.from) + context.unit;
                }
            }),
            language.foldNodeProp.add({
                "Block ClassBody SwitchBody EnumBody ObjectExpression ArrayExpression ObjectType": language.foldInside,
                BlockComment(tree) { return { from: tree.from + 2, to: tree.to - 2 }; }
            })
        ]
    }),
    languageData: {
        closeBrackets: { brackets: ["(", "[", "{", "'", '"', "`"] },
        commentTokens: { line: "//", block: { open: "/*", close: "*/" } },
        indentOnInput: /^\s*(?:case |default:|\{|\}|<\/)$/,
        wordChars: "$"
    }
});
const jsxSublanguage = {
    test: node => /^JSX/.test(node.name),
    facet: language.defineLanguageFacet({ commentTokens: { block: { open: "{/*", close: "*/}" } } })
};
/// A language provider for TypeScript.
const typescriptLanguage = javascriptLanguage.configure({ dialect: "ts" }, "typescript");
/// Language provider for JSX.
const jsxLanguage = javascriptLanguage.configure({
    dialect: "jsx",
    props: [language.sublanguageProp.add(n => n.isTop ? [jsxSublanguage] : undefined)]
});
/// Language provider for JSX + TypeScript.
const tsxLanguage = javascriptLanguage.configure({
    dialect: "jsx ts",
    props: [language.sublanguageProp.add(n => n.isTop ? [jsxSublanguage] : undefined)]
}, "typescript");
let kwCompletion = (name) => ({ label: name, type: "keyword" });
const keywords = "break case const continue default delete export extends false finally in instanceof let new return static super switch this throw true typeof var yield".split(" ").map(kwCompletion);
const typescriptKeywords = keywords.concat(["declare", "implements", "private", "protected", "public"].map(kwCompletion));
/// JavaScript support. Includes [snippet](#lang-javascript.snippets)
/// and local variable completion.
function javascript(config = {}) {
    let lang = config.jsx ? (config.typescript ? tsxLanguage : jsxLanguage)
        : config.typescript ? typescriptLanguage : javascriptLanguage;
    let completions = config.typescript ? typescriptSnippets.concat(typescriptKeywords) : snippets.concat(keywords);
    return new language.LanguageSupport(lang, [
        javascriptLanguage.data.of({
            autocomplete: autocomplete.ifNotIn(dontComplete, autocomplete.completeFromList(completions))
        }),
        javascriptLanguage.data.of({
            autocomplete: localCompletionSource
        }),
        config.jsx ? autoCloseTags : [],
    ]);
}
function findOpenTag(node) {
    for (;;) {
        if (node.name == "JSXOpenTag" || node.name == "JSXSelfClosingTag" || node.name == "JSXFragmentTag")
            return node;
        if (node.name == "JSXEscape" || !node.parent)
            return null;
        node = node.parent;
    }
}
function elementName(doc, tree, max = doc.length) {
    for (let ch = tree === null || tree === void 0 ? void 0 : tree.firstChild; ch; ch = ch.nextSibling) {
        if (ch.name == "JSXIdentifier" || ch.name == "JSXBuiltin" || ch.name == "JSXNamespacedName" ||
            ch.name == "JSXMemberExpression")
            return doc.sliceString(ch.from, Math.min(ch.to, max));
    }
    return "";
}
const android = typeof navigator == "object" && /Android\b/.test(navigator.userAgent);
/// Extension that will automatically insert JSX close tags when a `>` or
/// `/` is typed.
const autoCloseTags = view.EditorView.inputHandler.of((view, from, to, text, defaultInsert) => {
    if ((android ? view.composing : view.compositionStarted) || view.state.readOnly ||
        from != to || (text != ">" && text != "/") ||
        !javascriptLanguage.isActiveAt(view.state, from, -1))
        return false;
    let base = defaultInsert(), { state: state$1 } = base;
    let closeTags = state$1.changeByRange(range => {
        var _a;
        let { head } = range, around = language.syntaxTree(state$1).resolveInner(head - 1, -1), name;
        if (around.name == "JSXStartTag")
            around = around.parent;
        if (state$1.doc.sliceString(head - 1, head) != text || around.name == "JSXAttributeValue" && around.to > head) ;
        else if (text == ">" && around.name == "JSXFragmentTag") {
            return { range, changes: { from: head, insert: `</>` } };
        }
        else if (text == "/" && around.name == "JSXStartCloseTag") {
            let empty = around.parent, base = empty.parent;
            if (base && empty.from == head - 2 &&
                ((name = elementName(state$1.doc, base.firstChild, head)) || ((_a = base.firstChild) === null || _a === void 0 ? void 0 : _a.name) == "JSXFragmentTag")) {
                let insert = `${name}>`;
                return { range: state.EditorSelection.cursor(head + insert.length, -1), changes: { from: head, insert } };
            }
        }
        else if (text == ">") {
            let openTag = findOpenTag(around);
            if (openTag && openTag.name == "JSXOpenTag" &&
                !/^\/?>|^<\//.test(state$1.doc.sliceString(head, head + 2)) &&
                (name = elementName(state$1.doc, openTag, head)))
                return { range, changes: { from: head, insert: `</${name}>` } };
        }
        return { range };
    });
    if (closeTags.changes.empty)
        return false;
    view.dispatch([
        base,
        state$1.update(closeTags, { userEvent: "input.complete", scrollIntoView: true })
    ]);
    return true;
});

// This file was generated by lezer-generator. You probably shouldn't edit it.
const spec_identifier = {__proto__:null,true:22, false:22, null:24, super:48, this:50, if:114, else:116, while:120, for:124, var:131, let:133, const:135, assert:146, do:150, break:154, continue:158, return:162};
const parser = lr.LRParser.deserialize({
  version: 14,
  states: "<nQ`QPOOP#POPOOO$iQQO'#EcO&TQQO'#CcOOQO'#Cc'#CcOOQO'#Ef'#EfO&|QPO'#CkO'WQQO'#EaO*OQQO'#CpO*VQPO'#CoOOQO'#Ee'#EeO(kQQO'#EeO+YQQO'#CaO+aQPO'#DZO+hQPO'#DUO+mQPO'#D_O+mQPO'#DaOOQO'#Ea'#EaO,eQPO'#DdOOQO'#E`'#E`OOQO'#ER'#ERQ`QPOOO,jQQO'#CdO,oQPO'#DfO,oQPO'#DiO,tQPO'#DnO,|QPO'#DkO+mQPO'#DvO`QPO'#DxO-RQPO'#DzO-^QPO'#D|O-iQPO'#EOP-pOWO'#C^POOO)C?})C?}O-{QQO'#EcO.`QQO'#CcO/qQQO,59VO/xQPO'#EkO0TQPO,59rO+mQPO,59XO0YQPO,59OO+mQPO,58|O0hQPO'#CrOOQO,59Z,59ZOOQO'#Cz'#CzOOQO'#C}'#C}O+mQPO,59dO+mQPO,59dO+mQPO,59dO+mQPO,59dO+mQPO,59dO+mQPO,59dO+mQPO,59dO+mQPO,59dO+mQPO,59dO+mQPO,59vOOQO,59y,59yOOQO,58{,58{OOQO,59u,59uO0uQPO,59uO0|QPO,59pO1TQQO,59{O`QPO,5:OOOQO-E8P-E8PO0YQPO,59OO2bQPO'#CkO`QPO,5:QO`QPO,5:TOOQO'#DV'#DVO2lQPO'#EmO2tQPO'#DrO3PQPO,5:YO3UQPO'#DmO`QPO,5:VO`QPO,5:`O3lQQO,5:bO3vQPO,5:dOOQO'#De'#DeOOQO,5:f,5:fO3{QPO,5:fOOQO,5:h,5:hO4QQPO,5:hO4VQQO,5:jOOQO,5:j,5:jPOOO'#EQ'#EQP4^OWO,58xPOOO,58x,58xOOQO1G.q1G.qO,tQPO'#ETO4iQPO,5;VOOQO1G/^1G/^O4qQQO1G.sOOQO'#Ec'#EcO4xQQO1G.jO*VQPO1G.uOOQO1G.j1G.jO6fQQO1G.jO6kQQO1G.hO6xQQO'#EgO7VQPO,59^O8`QQO1G/OO9kQQO1G/OO9rQQO1G/OO:}QQO1G/OO;UQQO1G/OO<QQQO1G/OO<hQQO1G/OO=vQQO1G/OO?RQQO1G/OO?]QQO1G/bOOQO1G/a1G/aO?dQQO1G/[OOQO1G/[1G/[OOQO1G/j1G/jO?qQPO1G/lOOQO1G/o1G/oO,tQPO'#EVOAkQPO,5;XOAsQPO,5:^OOQO1G/t1G/tOBQQPO,5:aOBYQPO,5:XOBaQPO,5:XOOQO1G/q1G/qOOQO1G/z1G/zOOQO1G/|1G/|O+mQPO1G/|O,oQPO1G0OOOQO1G0Q1G0QOOQO1G0S1G0SOOQO1G0U1G0UPOOO-E8O-E8OPOOO1G.d1G.dOOQO,5:o,5:oOOQO-E8R-E8ROOQO7+$_7+$_OOQO7+$a7+$aOBfQPO7+$UO2bQPO'#ESOBqQPO,5;ROOQO1G.x1G.xO+mQPO7+$|O`QPO7+%WOOQO,5:q,5:qOOQO-E8T-E8TOB|QQO'#EnOCZQPO'#DsOOQO'#En'#EnOOQO1G/x1G/xO2bQPO1G/{OCkQQO1G/sO0hQPO1G/sOCrQPO1G/sOCyQQO7+%hODQQPO7+%jOOQO<<Gp<<GpODVQQO,5:nOOQO-E8Q-E8QODdQQO<<HhOOQO<<Hr<<HrODqQPO'#EoODyQPO,5:_OEOQQO7+%gO0hQPO7+%_OEVQPO7+%_OE[QQO7+%_OOQO<<IS<<ISOOQO<<IU<<IUOEcQPO'#EUOEvQPO,5;ZOOQO1G/y1G/yOOQO<<IR<<IROFOQPO<<HyOOQO<<Hy<<HyO0hQPO<<HyOOQO,5:p,5:pOOQO-E8S-E8SOOQOAN>eAN>eOFTQPOAN>eOOQOG24PG24POFYQQO'#EcOFjQQO'#CcO2bQPO,58|O2bQPO,59dO2bQPO,59dO2bQPO,59dO2bQPO,59dO2bQPO,59dO2bQPO,59dO2bQPO,59dO2bQPO,59dO2bQPO,59dOFzQPO,59pOGXQQO1G.hOHpQQO1G/OOJOQQO1G/OOJVQQO1G/OOKeQQO1G/OOKlQQO1G/OOLkQQO1G/OOMRQQO1G/OOMYQQO1G/OOMaQQO1G/[O2bQPO7+$|OMqQQO<<HhO! iQQO'#CpONRQQO'#EeOBTQPO'#DUO2bQPO'#D_O2bQPO'#DaO! pQQO1G/bO+mQPO,59v",
  stateData: "!!Q~O!|OS!}OSPOS#OPQ~OScOXTOYTOZTO[TO^UOhfOt`O|]O!S_O!U`O!V`O!ZgO!^hO!`jO!ciO!diO!eiO!kkO!mlO!omO!qnO!soO#URO#WQO~O#OpO~OS#VX^#VXb#VXe#VXj#VXk#VXm#VXo#VXp#VXr#VXs#VXt#VXu#VXv#VXw#VX!P#VX!S#VX#[#VX#]#VX#^#VX#`yX~O!Q!XX~P#UOSVX^VXbVXeVXjVXkVXmVXoVXpVXrVXsVXtVXuVXvVXwVX!PVX!SVX#[VX#]VX#^VX#`yX~O!Q!XX~P$pOXTOYTOZTO[TO^UOhfOt%iO!S%hO!U%iO!V%iO~O#UsO#WrO~P&[ObwOexOS#TXm#TXo#TXp#TXr#TXs#TXt#TXu#TXv#TXw#TX!P#TX!S#TX#[#TX#]#TX#^#TX]#TX!Q#TX`#TXg#TX~OjyOkyOS#XXb#XXe#XXm#XXo#XXp#XXr#XXs#XXt#XXu#XXv#XXw#XX!P#XX!S#XX#[#XX#]#XX#^#XX!Q#XX`#XX~O^dX~P(kO^zO~Om!OOo!POp!QOr!SOs!TOt!UOu!VOv!WOw!WO!P!XO!S!YO#[|O#]|O#^}O~OS!ZO~P*[O{![O~P`O#`!^O~OXTOYTOZTO[TO^UOhfOt`O!S_O!U`O!V`O#UsO#WrO~O!Q!`O~Oe!bO~O^!cO~O#U!fO#W!fO~O^!jO~OS!pO#U!oO#W!oO~OS!rO#U!oO#W!oO~OS!uO~P+mO#P!vO#Q!vO#R!xO~O]yX]#VXgyX!Q#VX`#VX~P#UO]VX]yXgyX!QVX`VX~P$pOm$}Oo%OOp%POr%ROs%SOt%TOu%UOv%VOw%VO!P%kO!S!YO#[|O#]|O#^}O~O]!yO~P.sOg!zO#`%WO]#_X~O]!|O~Oh#SOi#RO#USO#W#OO~O#U${O#W$zO]#ZP~P&[O{#bO~P`O|]O~P+mO!S!YOS!Tam!Tao!Tap!Tar!Tas!Tat!Tau!Tav!Taw!Ta!P!Ta#[!Ta#]!Ta#^!Ta]!Ta!Q!Ta`!Tag!Ta~O#U${O#W$zO~P&[Og#hOS#aX~Ok#jOS!fXg!fX~OS#kO~O!ciO!diO!eiO#U${O#W$zOS#ZP~P&[OS#qO!Q#rO~P*[O!^#sO~OS#tO~OS#uO~OS#vO~P*[O#P!vO#Q!vO#R#xO~Og!zO]#_a~O`#{O~P*[OSWi^dXbWieWijWikWimWioWipWirWisWitWiuWivWiwWi!PWi!SWi#[Wi#]Wi#^Wi]Wi!QWi`WigWi~Oe#}O~OSUi!QUi`Ui~P*[Og$OO]#ZXS#ZX~P.sO]$QO~Ou!VOv!WOw!WO!S!YOSlimlioliplirlisli!Pli#[li#]li#^li!Qli`li~Ot!UO~P7[Om!OOr!SOs!TOt!UOu!VOv!WOw!WO!S!YO#[|O#]|O#^}OSlipli!Pli!Qli`li~Ooli~P8gOo!PO~P8gOm!OOt!UOu!VOv!WOw!WO!S!YO#[|O#]|OSlioliplirlisli!Pli!Qli`li~O#^li~P9yO#^}O~P9yOu!VOv!WOw!WO!S!YOSlioliplisli!Pli!Qli`li~Om!OOr!SOt!UO#[|O#]|O#^}O~P;]Otli~P7[O!S!YOSlimlioliplirlislitlivliwli!Pli#[li#]li#^li]li`ligli~Ouli!Qli~P<oO!S!YOSlimlioliplirlislitlivliwli!Pli#[li#]li#^li`li~Ou!VO!Qli~P>QO!Q$RO~P*[OSxi!Qxi`xi~P*[O![$SOS!YiX!YiY!YiZ!Yi[!Yi^!Yih!Yit!Yi|!Yi!S!Yi!U!Yi!V!Yi!Z!Yi!^!Yi!`!Yi!c!Yi!d!Yi!e!Yi!k!Yi!m!Yi!o!Yi!q!Yi!s!Yi!z!Yi#U!Yi#W!Yi{!Yi~Og#hOS#aa~Ob$WO#U${O#W$zO~P&[O!Q$ZO#`%WO~OS$]O~P+mOS$^O~Oi$aO#USO#W#OO~Og$OO]#ZaS#Za~OS#bXg#bX`#bX~P.sOb$WO#U${O#W$zO`#cP~P&[OS$iO~P*[OS$iO~P+mOS$lO~P*[OS$mO~O]!vag!vaS!va~P.sOS!Oy!Q!Oy`!Oy~P*[Og$nO`#cX~O`$pO~O]$qO~P.sO]$sO~OS$tO~P*[Ob$WO#U${O#W$zO`!xXg!xX~P&[Og$nO`#ca~O]$wO~O]$yO~O]#VXg#VX!QyX`#VX~P#UO]VXgVX!QyX`VX~P$pO|]O#U${O#W$zO~P&[O]UigUiSUi`Ui~P.sOu%UOv%VOw%VO!S!YO]limlioliplirlisli!Pli#[li#]li#^ligliSli`li~Ot%TO~PGiOm$}Or%ROs%SOt%TOu%UOv%VOw%VO!S!YO#[|O#]|O#^}O]lipli!PligliSli`li~Ooli~PHwOo%OO~PHwOm$}Ot%TOu%UOv%VOw%VO!S!YO#[|O#]|O]lioliplirlisli!PligliSli`li~O#^li~PJ^O#^}O~PJ^Ou%UOv%VOw%VO!S!YO]lioliplisli!PligliSli`li~Om$}Or%ROt%TO#[|O#]|O#^}O~PKsOtli~PGiOu%UO~P<oO]xigxiSxi`xi~P.sO]!Oyg!OyS!Oy`!Oy~P.sOj$|Ok$|O]#XXb#XXe#XXm#XXo#XXp#XXr#XXs#XXt#XXu#XXv#XXw#XX!P#XX!S#XX#[#XX#]#XX#^#XXg#XXS#XX`#XX~O^dX~PNRO!Q%cO~P*[O!|!}YX#OP#Ou~",
  goto: ":Y#dPP#ePP#h#u%U&pPPPPPP(RP&pP)k*zP,^PPPPP#uP,dPP-WPPPPPP#u-x/fPP0w#uPP#uP#uPP#h1Y#hPP#hP#hP1m1pPPP2Q2W#h2]#hP#hP#hP#hP#hP2`2f2p2v2|3SPPPPPPPP3Y3uP6tP8W)k9gPPP9vP9y9|:VRqPgcO]el!]!`!d!e!k!l$S#kaOU]_`eklowyz!O!P!Q!R!S!T!U!V!W!X!]!^!`!c!d!e!j!k!l#j#m#r$O$R$S$W$Z$]$^$i$n$t$|$}%O%P%Q%R%S%T%U%V%W%c%h%i%k!dWO]_`eklowy!O!P!Q!R!S!T!U!V!W!X!]!^!`!d!e!k!l#m#r$R$S$^%kQ#PxQ#R!bQ$a#}!V%eUz!c!j#j$O$W$Z$]$i$n$t$|$}%O%P%Q%R%S%T%U%V%W%c%h%i!dZO]_`eklowy!O!P!Q!R!S!T!U!V!W!X!]!^!`!d!e!k!l#m#r$R$S$^%k!V%fUz!c!j#j$O$W$Z$]$i$n$t$|$}%O%P%Q%R%S%T%U%V%W%c%h%i#jYOU]_`eklowyz!O!P!Q!R!S!T!U!V!W!X!]!^!`!c!d!e!j!k!l#j#m#r$O$R$S$W$Z$]$^$i$n$t$|$}%O%P%Q%R%S%T%U%V%W%c%h%i%kQ!dgQ!ehR$`#s#kYOU]_`eklowyz!O!P!Q!R!S!T!U!V!W!X!]!^!`!c!d!e!j!k!l#j#m#r$O$R$S$W$Z$]$^$i$n$t$|$}%O%P%Q%R%S%T%U%V%W%c%h%i%k#jXOU]_`eklowyz!O!P!Q!R!S!T!U!V!W!X!]!^!`!c!d!e!j!k!l#j#m#r$O$R$S$W$Z$]$^$i$n$t$|$}%O%P%Q%R%S%T%U%V%W%c%h%i%kR#QxQ{XR#|#Qr!O[!m!t!}#T#X#Y#Z#[#]#a#c$[$_$d$k%jk$}t#U$V$b$h%X%Z%[%]%^%_%b%dp!R[!m!t!}#T#X#Y#[#]#a#c$[$_$d$k%ji%Qt#U$V$b$h%X%Z%[%^%_%b%d!d^O]_`eklowy!O!P!Q!R!S!T!U!V!W!X!]!^!`!d!e!k!l#m#r$R$S$^%kQuUS!hi#hQ#l!jQ#y!z!R%gz!c#j$O$W$Z$]$i$n$t$|$}%O%P%Q%R%S%T%U%V%W%c%h%i!d^O]_`eklowy!O!P!Q!R!S!T!U!V!W!X!]!^!`!d!e!k!l#m#r$R$S$^%k!V%gUz!c!j#j$O$W$Z$]$i$n$t$|$}%O%P%Q%R%S%T%U%V%W%c%h%ifcO]el!]!`!d!e!k!l$ST#d!^%WfbO]el!]!`!d!e!k!l$SQ!qmR!snR!kjfcO]el!]!`!d!e!k!l$SR#m!jQ!giR$T#hV$X#j$W$nR!ljQ!wpR#w!wQeOQ!]]T!ae!]Q$P#UR$c$PQ!{uR#z!{Q$o$fR$v$oQ#i!gR$U#iWdO]e!]Q!nlQ#e!`Q#f!dQ#g!eQ#o!kQ#p!lR$e$Sf[O]el!]!`!d!e!k!l$SStU!cS!Y_%hS!_`%iQ!mkQ!toQ!}wQ#TyY#Uz!j$]$i$tQ#W!OQ#X!PQ#Y!QQ#Z!RQ#[!SQ#]!TQ#^!US#_!V%UQ#`!WQ#a!XQ#c!^U$V#j$W$nQ$[#mQ$_#rQ$b$OQ$d$RQ$h$ZQ$k$^Q%X$|Q%Y$}Q%Z%OQ%[%PQ%]%QQ%^%RQ%_%SQ%`%TQ%a%VQ%b%WQ%d%cR%j%k#qSOU]_`eklowxyz!O!P!Q!R!S!T!U!V!W!X!]!^!`!b!c!d!e!j!k!l#j#m#r#}$O$R$S$W$Z$]$^$i$n$t$|$}%O%P%Q%R%S%T%U%V%W%c%h%i%k#kVOU]_`eklowyz!O!P!Q!R!S!T!U!V!W!X!]!^!`!c!d!e!j!k!l#j#m#r$O$R$S$W$Z$]$^$i$n$t$|$}%O%P%Q%R%S%T%U%V%W%c%h%i%kQ#VzQ#n!jQ$j$]Q$r$iR$x$tRvUR!iiQ$Y#jQ$f$WR$u$nR$g$W",
  nodeNames: "⚠ LineComment BlockComment Program ; ExpressionStatement AssignmentExpression Identifier FieldAccess IntegerLiteral FloatingPointLiteral BooleanLiteral null ) ( ParenthesizedExpression ] ArrayAccess [ MethodInvocation MethodName . ArgumentList , super this AssignOp AssignOp BinaryExpression CompareOp CompareOp LogicOp LogicOp BitOp BitOp BitOp ArithOp ArithOp ArithOp BitOp LambdaExpression Definition InferredParameters } { Block TernaryExpression LogicOp : UpdateExpression UpdateOp UnaryExpression LogicOp BitOp LabeledStatement Label IfStatement if else WhileStatement while ForStatement for ForSpec LocalVariableDeclaration var let const VariableDeclarator ArrayInitializer EnhancedForStatement ForSpec AssertStatement assert DoStatement do BreakStatement break ContinueStatement continue ReturnStatement return",
  maxTerm: 111,
  nodeProps: [
    ["isolate", -2,1,2,""],
    ["group", -14,4,5,45,54,56,59,61,64,70,72,74,76,78,80,"Statement",-15,6,7,8,9,10,11,12,15,17,19,28,40,46,49,51,"Expression"],
    ["openedBy", 13,"(",16,"[",43,"{"],
    ["closedBy", 14,")",18,"]",44,"}"]
  ],
  propSources: [highlight_js.highlighting],
  skippedNodes: [0,1,2,82],
  repeatNodeCount: 6,
  tokenData: "4i~R|XY#{YZ$pZ]#{]^$ppq#{qr%gtu%tuv&Yvw&gxy&wyz&|z{'R{|'Z|}'k}!O'p!O!P(T!P!Q*^!Q!R+Y!R![-O![!]1n!]!^1s!^!_1x!_!`2]!`!a2e!a!b2{!c!}3Q!}#O3f#P#Q3k#Q#R3p#R#S%t#T#o%t#o#p3x#p#q3}#q#r4_#r#s4d#y#z#{$f$g#{#BY#BZ#{$IS$I_#{$I|$I}$p$I}$JO$p$JT$JU#{$KV$KW#{&FU&FV#{~$QY!|~X^#{pq#{#y#z#{$f$g#{#BY#BZ#{$IS$I_#{$I|$JO#{$JT$JU#{$KV$KW#{&FU&FV#{~$wY!|~!}~X^#{pq#{#y#z#{$f$g#{#BY#BZ#{$IS$I_#{$I|$JO#{$JT$JU#{$KV$KW#{&FU&FV#{[%lP!US!_!`%oW%tOmW~%yT#U~tu%t!Q![%t!c!}%t#R#S%t#T#o%t~&_Pv~!_!`&b~&gOj~~&lQ#^~vw&r!_!`&b~&wOo~~&|O^~~'RO]~~'WPu~!_!`&b~'`Qt~{|'f!_!`&b~'kO!S~~'pOg~~'uRt~}!O'f!_!`&b!`!a(O~(TO#`~[(YPeW!Q![(]S(bWYS!Q![(]!f!g(z!g!h)P!h!i(z#R#S*T#W#X(z#X#Y)P#Y#Z(zS)POYSS)SR{|)]}!O)]!Q![)cS)`P!Q![)cS)hUYS!Q![)c!f!g(z!h!i(z#R#S)z#W#X(z#Y#Z(zS)}Q!Q![)c#R#S)zS*WQ!Q![(]#R#S*T~*cRu~z{*l!P!Q*q!_!`&b~*qO#O~~*vSP~OY*qZ;'S*q;'S;=`+S<%lO*q~+VP;=`<%l*q~+_aX~!O!P,d!Q![-O!d!e.U!f!g(z!g!h)P!h!i(z!n!o-v!q!r.s!z!{/[#R#S-{#U#V.U#W#X(z#X#Y)P#Y#Z(z#`#a-v#c#d.s#l#m/[S,iVYS!Q![(]!f!g(z!g!h)P!h!i(z#W#X(z#X#Y)P#Y#Z(z~-TZX~!O!P,d!Q![-O!f!g(z!g!h)P!h!i(z!n!o-v#R#S-{#W#X(z#X#Y)P#Y#Z(z#`#a-v~-{OX~~.OQ!Q![-O#R#S-{~.XQ!Q!R._!R!S._~.dTX~!Q!R._!R!S._!n!o-v#R#S.U#`#a-v~.vP!Q!Y.y~/OSX~!Q!Y.y!n!o-v#R#S.s#`#a-v~/_S!O!P/k!Q![0^!c!i0^#T#Z0^S/nR!Q![/w!c!i/w#T#Z/wS/zU!Q![/w!c!i/w!r!s)P#R#S/k#T#Z/w#d#e)P~0cXX~!O!P1O!Q![0^!c!i0^!n!o-v!r!s)P#R#S1b#T#Z0^#`#a-v#d#e)PS1RT!Q![/w!c!i/w!r!s)P#T#Z/w#d#e)P~1eR!Q![0^!c!i0^#T#Z0^~1sO!Q~~1xOS~~1}Q#[~!^!_2T!_!`%o~2YPw~!_!`&b~2bPk~!_!`%o~2jQ#]~!_!`%o!`!a2p~2uQw~!_!`&b!`!a2T~3QO!P~~3VT#W~tu3Q!Q![3Q!c!}3Q#R#S3Q#T#o3Q~3kOb~~3pO`~~3uPs~!_!`&b~3}O|~~4SQr~!_!`&b#p#q4Y~4_Op~~4dO{~~4iO!V~",
  tokenizers: [2, 3, new lr.LocalTokenGroup("SORO~", 3), new lr.LocalTokenGroup("j~RQYZXz{^~^O#Q~~aP!P!Qd~iO#R~~", 25, 93)],
  topRules: {"Program":[0,3]},
  dynamicPrecedences: {"99":-1},
  specialized: [{term: 98, get: (value) => spec_identifier[value] || -1}],
  tokenPrec: 2155
});

const jexlLanguage = language.LRLanguage.define({

    name: "jexl",

    parser: parser.configure({

        props: [

            language.indentNodeProp.add({

                IfStatement: language.continuedIndent({ except: /^\s*({|else\b)/ }),

                TryStatement: language.continuedIndent({ except: /^\s*({|catch\b|finally\b)/ }),

                LabeledStatement: language.flatIndent,

                Block: language.delimitedIndent({ closing: "}" }),

                ArrowFunction: cx => cx.baseIndent + cx.unit,

                "TemplateString BlockComment": () => null,

                "Statement Property": language.continuedIndent({ except: /^{/ }),

            }),

            language.foldNodeProp.add({

                "Block ArrayExpression": language.foldInside,

                BlockComment(tree) { return { from: tree.from + 2, to: tree.to - 2 }; }

            })

        ]

    }),

    languageData: {

        closeBrackets: { brackets: ["(", "[", "{", "'", '"', "`"] },

        commentTokens: { line: "//", block: { open: "/*", close: "*/" } },

        indentOnInput: /^\s*(?:case |default:|\{|\}|<\/)$/,

        wordChars: "$"

    }

});

//exports.jexl = jexl();

//exports.javascriptLanguage = jexlLanguage;

/// Connects an [ESLint](https://eslint.org/) linter to CodeMirror's
/// [lint](#lint) integration. `eslint` should be an instance of the
/// [`Linter`](https://eslint.org/docs/developer-guide/nodejs-api#linter)
/// class, and `config` an optional ESLint configuration. The return
/// value of this function can be passed to [`linter`](#lint.linter)
/// to create a JavaScript linting extension.
///
/// Note that ESLint targets node, and is tricky to run in the
/// browser. The
/// [eslint-linter-browserify](https://github.com/UziTech/eslint-linter-browserify)
/// package may help with that (see
/// [example](https://github.com/UziTech/eslint-linter-browserify/blob/master/example/script.js)).
function esLint(eslint, config) {
    if (!config) {
        config = {
            parserOptions: { ecmaVersion: 2019, sourceType: "module" },
            env: { browser: true, node: true, es6: true, es2015: true, es2017: true, es2020: true },
            rules: {}
        };
        eslint.getRules().forEach((desc, name) => {
            if (desc.meta.docs.recommended)
                config.rules[name] = 2;
        });
    }
    return (view) => {
        let { state } = view, found = [];
        for (let { from, to } of jexlLanguage.findRegions(state)) {
            let fromLine = state.doc.lineAt(from), offset = { line: fromLine.number - 1, col: from - fromLine.from, pos: from };
            for (let d of eslint.verify(state.sliceDoc(from, to), config))
                found.push(translateDiagnostic(d, state.doc, offset));
        }
        return found;
    };
}
function mapPos(line, col, doc, offset) {
    return doc.line(line + offset.line).from + col + (line == 1 ? offset.col - 1 : -1);
}
function translateDiagnostic(input, doc, offset) {
    let start = mapPos(input.line, input.column, doc, offset);
    let result = {
        from: start,
        to: input.endLine != null && input.endColumn != 1 ? mapPos(input.endLine, input.endColumn, doc, offset) : start,
        message: input.message,
        source: input.ruleId ? "eslint:" + input.ruleId : "eslint",
        severity: input.severity == 1 ? "warning" : "error",
    };
    if (input.fix) {
        let { range, text } = input.fix, from = range[0] + offset.pos - start, to = range[1] + offset.pos - start;
        result.actions = [{
                name: "fix",
                apply(view, start) {
                    view.dispatch({ changes: { from: start + from, to: start + to, insert: text }, scrollIntoView: true });
                }
            }];
    }
    return result;
}

exports.autoCloseTags = autoCloseTags;
exports.completionPath = completionPath;
exports.esLint = esLint;
exports.javascript = javascript;
exports.javascriptLanguage = javascriptLanguage;
exports.jsxLanguage = jsxLanguage;
exports.localCompletionSource = localCompletionSource;
exports.scopeCompletionSource = scopeCompletionSource;
exports.snippets = snippets;
exports.tsxLanguage = tsxLanguage;
exports.typescriptLanguage = typescriptLanguage;
exports.typescriptSnippets = typescriptSnippets;
